/* The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code, released March
 * 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation. Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 * 
 */
/**
    File Name:          7.7.2.js
    ECMA Section:       7.7.2 Boolean Literals

    Description:        BooleanLiteral::
                        true
                        false

                        The value of the Boolean literal true is a value of the
                        Boolean type, namely true.

                        The value of the Boolean literal false is a value of the
                        Boolean type, namely false.

    Author:             christine@netscape.com
    Date:               16 september 1997
*/

    var SECTION = "7.7.2";
    var VERSION = "ECMA_1";
    startTest();
    var TITLE   = "Boolean Literals";

    writeHeaderToLog( SECTION + " "+ TITLE);

    var testcases = getTestCases();
    test();

function getTestCases() {
    var array = new Array();
    var item = 0;

    // StringLiteral:: "" and ''

    array[item++] = new TestCase( SECTION, "true",     Boolean(true),     true );
    array[item++] = new TestCase( SECTION, "false",    Boolean(false),    false );

    return ( array );
}
function test() {
        for ( tc=0; tc < testcases.length; tc++ ) {
            testcases[tc].actual = testcases[tc].actual;

            testcases[tc].passed = writeTestCaseResult(
                            testcases[tc].expect,
                            testcases[tc].actual,
                            testcases[tc].description +" = "+ testcases[tc].actual );

            testcases[tc].reason += ( testcases[tc].passed ) ? "" : "wrong value ";

        stopTest();
        }
        return ( testcases );
}
